//
//  LockController.m
//  AsReaderScan
//
//  Created by Mikiko on 11/7/24.
//  Copyright © 2024 SPS INC. All rights reserved.
//

#import "LockController.h"
#import "AsUtils.h"
#import "ModelSettings.h"
#import "GlobalDefine.h"
#import "Utils.h"
#import "ErrorCodeHelper.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>

@interface LockController ()<AsReaderRFIDDeviceDelegate, AsReaderDeviceDelegate>


@property (weak, nonatomic) IBOutlet UITextField *olTargetEpc;
@property (weak, nonatomic) IBOutlet UITextField *olAccessPassword;
@property (weak, nonatomic) IBOutlet UISegmentedControl *olTargetMemory;
@property (weak, nonatomic) IBOutlet UISegmentedControl *olAction;
@end

@implementation LockController

#pragma mark - Lifecycle
//----------------------------------------------------------------
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view.
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    [AsReaderCommonDevice sharedInstance].delegateRFID = self;
    [AsReaderCommonDevice sharedInstance].delegateDevice = self;
    
    NSDictionary *dic = [[ModelSettings sharedInstance].m_arReadTagData objectAtIndex:[ModelSettings sharedInstance].m_nSelectTagNum];
    self.olTargetEpc.text = [AsUtils getEpcStringFromPcEPCData:[dic objectForKey:df_CELL_TAG_RAW]];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [self setOlTargetEpc:nil];
    [self setOlAccessPassword:nil];
    [self setOlTargetMemory:nil];
    [self setOlAction:nil];
    [AsReaderCommonDevice sharedInstance].delegateRFID = nil;
    [AsReaderCommonDevice sharedInstance].delegateDevice = nil;
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    [self.view endEditing:YES];
}

#pragma mark - AsReaderRFIDDeviceDelegate
//----------------------------------------------------------------
- (void)lockedReceived:(int)statusCode
{
    if(statusCode == 0x00) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [Utils showAlert:@"Confirm" message:@"Lock Tag." view:self];
       });
    }
}


- (void)errorReceived:(NSData *)errorCode{
    
    ErrorCodeHelper *errorHelper = [ErrorCodeHelper sharedInstance];
    NSString *errorResult = [errorHelper errorReceived:errorCode];
    
        dispatch_async(dispatch_get_main_queue(), ^{
            [Utils showAlert:@"" message:errorResult view:self];
        });
}

- (void)plugged:(BOOL)plug { 
    
}


- (void)readerConnected:(int)status { 
    
}


#pragma mark - IBAction
//----------------------------------------------------------------
- (IBAction)rightBarButtonItemClicked:(id)sender
{
    [self.olTargetEpc resignFirstResponder];
    
    NSString *hexaString = self.olAccessPassword.text;
    NSScanner *pScanner = [NSScanner scannerWithString: hexaString];
    uint32_t accesspassword;
    [pScanner scanHexInt: &accesspassword];
    
    uint32_t lock = 0;
    uint32_t seed = (uint32_t)self.olAction.selectedSegmentIndex;
    
    switch (self.olTargetMemory.selectedSegmentIndex) {
        case 0: // Kill
            lock = (seed << 8) | (3 << 18);
            break;
        case 1: // Access
            lock = (seed << 6) | (3 << 16);
            break;
        case 2: // EPC
            lock = (seed << 4) | (3 << 14);
            break;
        case 3: // TID
            lock = (seed << 2) | (3 << 12);
            break;
        case 4: // USER
            lock = (seed << 0) | (3 << 10);
            break;
        default:
            break;
    }
    
    NSData *epcs = [self.olTargetEpc.text hexStringToBytes];
    [[AsReaderRFIDDevice sharedInstance] lockTagMemoryWithAccessPassword:accesspassword epc:epcs lockData:lock];
}

- (void)encodeWithCoder:(nonnull NSCoder *)coder { 
    
}

- (void)traitCollectionDidChange:(nullable UITraitCollection *)previousTraitCollection { 
    
}

- (void)preferredContentSizeDidChangeForChildContentContainer:(nonnull id<UIContentContainer>)container { 
    
}

- (CGSize)sizeForChildContentContainer:(nonnull id<UIContentContainer>)container withParentContainerSize:(CGSize)parentSize { 
    return [super sizeForChildContentContainer:container withParentContainerSize:parentSize];
}

- (void)systemLayoutFittingSizeDidChangeForChildContentContainer:(nonnull id<UIContentContainer>)container { 
    
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(nonnull id<UIViewControllerTransitionCoordinator>)coordinator { 
    
}

- (void)willTransitionToTraitCollection:(nonnull UITraitCollection *)newCollection withTransitionCoordinator:(nonnull id<UIViewControllerTransitionCoordinator>)coordinator { 
    
}

- (void)didUpdateFocusInContext:(nonnull UIFocusUpdateContext *)context withAnimationCoordinator:(nonnull UIFocusAnimationCoordinator *)coordinator { 
    
}

- (void)setNeedsFocusUpdate { 
    
}

- (BOOL)shouldUpdateFocusInContext:(nonnull UIFocusUpdateContext *)context { 
    return [super shouldUpdateFocusInContext:context];
}

- (void)updateFocusIfNeeded { 
    
}

@end
